#include <12F683.h>

// Bits de configuracin
#FUSES WDT				// Watch Dog
#FUSES INTRC_IO		// Oscilador interno
#FUSES MCLR				// Pin usado para reset
#FUSES PUT				// Power Up Timer
#FUSES PROTECT			// Cdigo protegido
#FUSES CPD				// EEPROM Protegida
#FUSES BROWNOUT		// Brownout
#FUSES NOIESO			// Cambio de oscilador interno-externo desactivado
#FUSES NOFCMEN			// Monitorizacin de clock desactivada

#use delay(int=1000000,RESTART_WDT)		// Frecuencia de 1 MHz

// Definiciones
#define PUERTA_1	PIN_A1
#define LARGAS		PIN_A2
#define PUERTA_2	PIN_A4
#define PUERTA_3	PIN_A5
#define T_BOTON_PULSADO				2500	// Tiempo que se mantienen cerrados los rels (ms)
#define T_MAXIMO_RAFAGA				1000	// Tiempo mximo de una pulsacin para que sea reconocida como rfaga y no como luz de carretera (ms)
#define T_ANTIRREBOTES				50		// Tiempo mnimo de una pulsacin para que sea tenida en cuenta (ms)
#define T_ESPERA_ENTRE_RAFAGAS	700	// Tiempo mximo de espera entre rfaga y rfaga (ms)

// Declaracin de variables
short pulsacion;
int contador_pulsaciones;
long t_restante;

#int_EXT									// Interrupcin externa (cuando se pulsan las rfagas o la luz de carretera)
void  Interrupcion_externa() {
	pulsacion=1;
}

#int_TIMER1								// Interrupcin Timer1 para resetear WatchDog
void  TIMER1() {
	restart_wdt();
}

void contar_pulsaciones() {		// Subrutina que cuenta el tiempo que estn las largas encendidas
	long contador_largas_encendidas;

	delay_ms(T_ANTIRREBOTES);																		// Retardo para minimizar los rebotes
	if (input(LARGAS)==1) {																			// Si an estn encendidas proceder, si no es que era un rebote
		contador_largas_encendidas=0;																// Inicializar un contador de la duracin de la rfaga
		while ((input(LARGAS)==1)&&(contador_largas_encendidas<T_MAXIMO_RAFAGA)) {	// Mientras las largas estn encendidas incrementar el contador
			contador_largas_encendidas++;
			delay_ms(1);
			}
		if (contador_largas_encendidas>=T_MAXIMO_RAFAGA)									// Si se ha llegado al tiempo mximo de rfaga es que eran las luces de carretera
			contador_pulsaciones=0;																	// Se resetea el contador de pulsaciones
		else
			contador_pulsaciones++;																	// En caso contrario incrementar el contador de pulsaciones
		delay_ms(T_ANTIRREBOTES);
	}
	pulsacion=0;																						// Se resetea el flag
}

// Programa principal
void main() {
	output_low(PUERTA_1);			// Inicializacin de las salidas
	output_low(PUERTA_2);
	output_low(PUERTA_3);
	pulsacion=0;						// Inicializacin de variables
	contador_pulsaciones=0;

	setup_wdt(WDT_ON);								// Activacin del WatchDog
	setup_wdt(WDT_1152MS|WDT_TIMES_4);			// Reset por WathDog cada 4.6 s
	setup_timer_1(T1_INTERNAL|T1_DIV_BY_4);	// Interrupcin de Timer1 cada 1 s

	enable_interrupts(INT_TIMER1);		// Activar interrupcin de Timer1 para reseteo del WatchDog
	enable_interrupts(INT_EXT);			// Activar interrupcin externa
	ext_int_edge(L_TO_H);					// Interrupcin externa activada por flanco de subida
	enable_interrupts(GLOBAL);				// Activar interrupciones

	while(TRUE) {					// Bucle del programa principal
		if (pulsacion==1)							// Si salta el flag de interrupcin iniciar la subrutina
			contar_pulsaciones();

		t_restante=T_ESPERA_ENTRE_RAFAGAS;							// Bucle de espera para recibir una segunda rfaga
		while ((contador_pulsaciones==1)&&(t_restante>0)) {
			if (pulsacion==1)
				contar_pulsaciones();
			delay_ms(1);
			t_restante--;
		}

		t_restante=T_ESPERA_ENTRE_RAFAGAS;							// Bucle de espera para recibir una tercera rfaga
		while ((contador_pulsaciones==2)&&(t_restante>0)) {
			if (pulsacion==1)
				contar_pulsaciones();
			delay_ms(1);
			t_restante--;
		}

		t_restante=T_ESPERA_ENTRE_RAFAGAS;							// Bucle de espera para comprobar que no hay una cuarta rfaga
		while ((contador_pulsaciones==3)&&(t_restante>0)) {
			if (pulsacion==1)
				contar_pulsaciones();
			delay_ms(1);
			t_restante--;
		}

		if (contador_pulsaciones!=0) {			// Si ha habido alguna rfaga proceder
			disable_interrupts(INT_EXT);			// Desactivacin de la interrupcin externa
			switch (contador_pulsaciones) {		// Dependiendo del nmero de rfagas cerrar uno u otro rel
				case 1:output_high(PUERTA_1);
						delay_ms(T_BOTON_PULSADO);
						output_low(PUERTA_1);
						contador_pulsaciones=0;
						break;
				case 2:output_high(PUERTA_2);
						delay_ms(T_BOTON_PULSADO);
						output_low(PUERTA_2);
						contador_pulsaciones=0;
						break;
				case 3:output_high(PUERTA_3);
						delay_ms(T_BOTON_PULSADO);
						output_low(PUERTA_3);
						contador_pulsaciones=0;
						break;
				default:contador_pulsaciones=0;
						break;
			}
			enable_interrupts(INT_EXT);			// Activacin de la interrupcin externa
		}
	}
}